
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    print qq{1..0 # SKIP these tests are for testing by the author\n};
    exit
  }
}

use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;

set_spell_cmd('aspell list');
add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
API
Boggs
CPAN
DAW
Dragadiddle
Drummer
Euclidean
Euclidian
Flam
Flamacue
Flammed
Gene
Goroway
Jazz
MIDI
OO
Paradiddle
Pataflafla
Ratamacue
Rock
Rudd
Rudiments
SIXTYFOURTH
Simple
THIRTYSECOND
TODO
backbeat
constuctor
controled
de
drumkit
euclid
facto
fff
filename
flam
flamacue
flammed
gene
hhat
lib
pataflafla
reverb
